

import java.text.DecimalFormat;

/**
 * ITI 1220 Assignment 7, Question 3
 * Sample solution
 * 
 * @author Alan Williams
 */
public class A7Q3
{

	/**
	 * Main method to determine the steady state temperatures of a metal plate with
	 * refrigerated edges and a heat source applied internally.
	 * 
	 * @param args
	 */
	public static void main( String[] args )
	{
		// DECLARE VARIABLES / DATA DICTIONARY
		
		double[][] plate;		// Temperature values at sample points on a hot metal plate.
		double[][] platePrime;	// Temperature values after averaging algorithm.
		double maxDiff;			// Maximum change in corresponding values from plate to platePrime.

		DecimalFormat df = new DecimalFormat( "#.##" );	// Used to format difference value.
		
		// BODY OF ALGORITHM
		
		// Set up initial temperature values.

		plate = A7Q3.initialMatrix( );

		// Print intial temperature values
		
		A7Q3.printMatrix( plate );

		// As long as the old and new temperature values appear to be changing, continue
		// updating the temperatures.
		
		maxDiff = 1.0;
		while ( maxDiff > 0.01 )
		{
			// Do one iteration of the averaging algorithm to get new temperature values.
			
			platePrime = A7Q3.avgTempIterate( plate );
			
			// Determine the maximum change in the temperature values.
			
			maxDiff = A7Q3.findMaxDiff( plate, platePrime );
			
			// Print the difference
			
			System.out.println( "Maximum change is " + df.format( maxDiff ) );
			
			// Repeat with the new temperature values.
			
			plate = platePrime;
		}

		// At this point, the maximum temperature change is less than 0.01.  Print
		// the "steady state" temperatures.
		
		A7Q3.printMatrix( plate );

	}

	/**
	 * Performs one iteration of the temperature update for the hot metal plate.
	 * <p>
	 * A new matrix is created and set up with the values 20.0 at all edges.  Then,
	 * for each value in the original matrix, the average of the values that are one
	 * position up, down, left, and right is used as a new temperature estimated value.
	 * <p>
	 * Note that the updated values are kept separate from the original values, so that
	 * the new values do not affect nearby averages.
	 * 
	 * @param origMatrix Matrix of hot plate temperature values.
	 * @return A new matrix with the same dimensions, with updated temperature values.
	 */
	public static double[][] avgTempIterate( double[][] origMatrix )
	{
		// DECLARE VARIABLES / DATA DICTIONARY
		
		double[][] newMatrix; 	// RESULT: updated matrix

		int row; 				// Row index for matrices
		int col; 				// Column index for matrices

		// BODY OF ALGORITHM
		
		// Create new matrix to receive averaged values.

		newMatrix = initialMatrix( );

		// Go through each interior matrix value. Note that indices start at 1
		// instead of 0, and go up only to the second last array position
		// instead of the last position. Therefore, the edge values remain
		// unchanged.

		for ( row = 1; row < origMatrix.length - 1; row++ )
		{
			for ( col = 1; col < origMatrix[row].length - 1; col++ )
			{
				// The new matrix gets the average of the values to the left,
				// right, up, and down, from the corresponding value in the 
				// original matrix.

				newMatrix[row][col] = 0.25 * ( origMatrix[row - 1][col]
						+ origMatrix[row + 1][col] + origMatrix[row][col - 1] + origMatrix[row][col + 1] );
			}
		}

		// Reset the "hot spot" to its fixed value.

		newMatrix[2][7] = 100.0;

		// RETURN RESULT
		
		return newMatrix;
	}

	/**
	 * Method to initialize the heat plate problem.
	 * 
	 * @return A 10 x 10 matrix with all interior values set to 50.0, all edge
	 *         values set to 20.0, and location [2][7] set to 100.0
	 */
	public static double[][] initialMatrix( )
	{
		// DECLARE VARIABLES / DATA DICTIONARY

		double[][] result; 	// RESULT: initialized matrix

		int numRows; 		// Number of rows in matrix
		int numCols; 		// Number of columns in matrix
		int row; 			// Row index in matrix
		int col; 			// Column index in matrix
		int hotRow; 		// Row index of hot spot
		int hotCol; 		// Column index of hot spot
		double edgeTemp; 	// Fixed edge temperature
		double regTemp; 	// Initial interior temperature
		double hotTemp; 	// Fixed hot spot temperature

		// BODY OF ALGORITHM

		// For this problem, the parameters' values are hard-coded.

		numRows = 10;
		numCols = 10;
		hotRow = 2;
		hotCol = 7;
		edgeTemp = 20.0;
		hotTemp = 100.0;
		regTemp = 50.0;

		// Create result matrix.

		result = new double[numRows][numCols];

		// Fill in interior temperature values. Note that indices start at 1
		// instead of 0, and go up only to the second last array position
		// instead of the last position.

		for ( row = 1; row < numRows - 1; row++ )
		{
			for ( col = 1; col < numCols - 1; col++ )
			{
				result[row][col] = regTemp;
			}
		}

		// Fill in edge temperature values.

		// For each row, set first and last columns:

		for ( row = 0; row < numRows; row++ )
		{
			result[row][0] = edgeTemp;
			result[row][numCols - 1] = edgeTemp;
		}

		// For each column, set first and last rows:

		for ( col = 0; col < numCols; col++ )
		{
			result[0][col] = edgeTemp;
			result[numRows - 1][col] = edgeTemp;
		}

		// Set hot spot temperature.

		result[hotRow][hotCol] = hotTemp;

		// RETURN RESULT

		return result;
	}

	/**
	 * Returns the maximum difference between corresponding values in two
	 * matrices.
	 * <p>
	 * The two matrices are assumed to have the same dimensions.
	 * 
	 * @param oldMatrix
	 *            Matrix before most recent averaging
	 * @param newMatrix
	 *            Matrix after most recent averaging
	 * @return Absolute value of the maximum difference between corresponding
	 *         matrix values.
	 */
	public static double findMaxDiff( double[][] oldMatrix, double[][] newMatrix )
	{
		double maxDiff; // RESULT: Maximum difference seen so far
		double currentDiff; // Difference between current array position values
		int row; // Matrix row index
		int col; // Matrix column index

		// Start the largest difference at a small value, which will presumably
		// be increased.

		maxDiff = 0.0;
		for ( row = 0; row < oldMatrix.length; row++ )
		{
			for ( col = 0; col < oldMatrix[row].length; col++ )
			{
				// Compare values between two matrices.

				currentDiff = Math.abs( oldMatrix[row][col]
						- newMatrix[row][col] );

				// See if we have found a larger difference.

				if ( currentDiff > maxDiff )
				{

					maxDiff = currentDiff;
				}
				else
				{
					; // do nothing
				}
			}
		}
		return maxDiff;
	}

	/**
	 * Prints a matrix of double values, formatted so that each value has 3
	 * digits and 2 decimal places.
	 * 
	 * @param aMatrix
	 *            Matrix to print.
	 */
	public static void printMatrix( double[][] aMatrix )
	{
		// DECLARE VARIABLES / DATA DICTIONARY

		DecimalFormat df = new DecimalFormat( "000.00" ); // Used to format
		// double values.
		int row; // Row index
		int col; // Column index

		// BODY OF ALGORITHM

		for ( row = 0; row < aMatrix.length; row++ )
		{
			// Print each row on one line (note "print" instead of "println")

			for ( col = 0; col < aMatrix[row].length; col++ )
			{
				System.out.print( df.format( aMatrix[row][col] ) + " " );
			}

			// Add new line at end of each row.

			System.out.println( );
		}
	}

}
