

/**
 * ITI 1220 Assignment 7, Question 1 Sample solution
 * 
 * @author Alan Williams
 */
public class BinarySearch
{
	/**
	 * Starter method for recursive binary search of a sorted array of integers.
	 * <p>
	 * This method is used to start the recursion, by making the first call to
	 * the recursive method, and adding the parameter values 0 and the length of
	 * the array (minus 1), indicating that the entire array should be searched.
	 * <p>
	 * The value returned by the recursive method is passed through as the
	 * return value of this method.
	 * 
	 * @param valueList
	 *            A sorted array of integers.
	 * @param findMe
	 *            The value for which to search in the array.
	 * @return True if the value is contained in the array, and false otherwise.
	 */
	public static boolean search( int[] valueList, int findMe )
	{
		return searchRec( valueList, findMe, 0, valueList.length - 1 );
	}

	/**
	 * Recursive binary search of a sorted array of integers.
	 * <p>
	 * This method will search between the specified array locations (inclusive)
	 * for a specified value.
	 * <p>
	 * If the distance between the left index and the right is "small", the
	 * result is returned directly. Otherwise, the value to find is compared
	 * with the "middle" value of the array. If the comparison is not equal,
	 * then either the left or the right half of the arrray is excluded from the
	 * search, and the other half of the array is searched recursively.
	 * 
	 * @param valueList
	 *            A sorted array of integers to be searched.
	 * @param findMe
	 *            The value to find in the array.
	 * @param leftIndex
	 *            The leftmost array index to be included in the search.
	 * @param rightIndex
	 *            The rightmost array index to be included in the search. This
	 *            is assumed to be less than the length of the array.
	 * @return True if the value is contained in the array, and false otherwise.
	 */
	public static boolean searchRec( int[] valueList, int findMe,
			int leftIndex, int rightIndex )
	{
		// DECLARE VARIABLES / DATA DICTIONARY

		boolean found; 	// RESULT: True if search is successful, and false
						// otherwise.
		int mid; 		// Index of array closest to the midpoint between leftIndex
						// and rightIndex

		// BODY OF ALGORITHM

		// Check for base case.

		// The base case covers 2 situations: leftIndex and rightIndex are the
		// same, or they are two consecutive array positions. The latter case is
		// needed asc there is no useful midpoint between two consecutive array
		// positions, and the possibility of not reducing the size of the interval.

		if ( leftIndex + 1 >= rightIndex )
		{
			// For the base case, if the value doesn't match one of the two
			// possible endpoints, the value is not in the array.

			found = findMe == valueList[leftIndex]
					|| findMe == valueList[rightIndex];
		}
		else
		{
			// Determine array position closest to the midpoint between
			// leftIndex and rightIndex.

			mid = ( leftIndex + rightIndex ) / 2;

			// Compare with value at midpoint.

			if ( findMe == valueList[mid] )
			{
				// We got lucky and found the value.

				found = true;
			}
			else
			{
				// Decide whether the value, if it were present, would be to the
				// left of the midpoint or to the right of the midpoint.

				if ( findMe < valueList[mid] )
				{
					// Value is on left side of midpoint. Search left half of
					// array recursively.

					found = searchRec( valueList, findMe, 0, mid );
				}
				else
				{
					// Value is on the right side of the midpoint. Search right
					// half of array recursively.

					found = searchRec( valueList, findMe, mid, rightIndex );
				}
			}
		}

		// RETURN RESULT

		return found;
	}

}
